# coding: utf-8

'''
    bearychat
    ~~~~~~~~~

    A simple package for interacting with `bearychat`_ API.

    .. _bearychat: http://bearychat.com/
'''

from __future__ import absolute_import

from .incoming import Incoming, InvalidPayloadError


__all__ = ['Incoming', 'InvalidPayloadError']



# coding: utf-8

'''
    incoming
    ~~~~~~~~

    Incoming robot api.
'''

__all__ = ['InvalidPayloadError', 'Incoming']


import requests


class InvalidPayloadError(ValueError):
    pass


class Incoming(object):
    '''Incoming api.

        resp = Incoming(hook_url) \
            .to('bearychat-dev') \
            .with_text('hello, world', markdown=False) \
            .with_attchment(title='pic', text='cool picture') \
            .push()

        resp = Incoming(hook_url) \
            .to('bearychat-dev') \
            .with_markdown('# hello, world\n## foobar') \
            .push()
    '''

    DEFAULT_CHANNEL = None

    def __init__(self, hook):
        # hook url
        self.hook = hook

        self.reset()

    def reset(self):
        '''Reset stream.'''
        self._text = None
        self._markdown = False
        self._channel = Incoming.DEFAULT_CHANNEL
        self._attachments = []

        return self

    def to(self, channel):
        '''Set destinate channel.

        :param channel: channel name.
        '''
        self._channel = channel

        return self

    def with_text(self, text, markdown=None):
        '''Set text content.

        :param text: text content.
        :param markdown: is markdown? Defaults to ``False``.
        '''
        self._text = text
        self._markdown = markdown or False

        return self

    def with_markdown(self, text):
        '''Set markdown content.

        :param text: markdown text content.
        '''
        return self.with_text(text, markdown=True)

    def with_attachment(self, **payload):
        '''Add an attachment.

        :param payload: attachment payload.
        '''
        self._attachments.append(payload)

        return self

    def build_message(self):
        '''Build a message.

        :raises InvalidPayloadError:
        '''
        if self._text is None:
            raise InvalidPayloadError('text is required')

        message = {
            'text': self._text,
            'markdown': self._markdown
        }

        if self._channel != Incoming.DEFAULT_CHANNEL:
            message['channel'] = self._channel

        if self._attachments:
            message['attachments'] = []
            for attachment in self._attachments:
                if 'title' not in attachment and 'text' not in attachment:
                    raise InvalidPayloadError('title or text is required')
                message['attachments'].append(attachment)

        return message

    def push(self):
        '''Deliver the message.'''
        message = self.build_message()
        return requests.post(self.hook, json=message)



# coding: utf-8

import pytest
import responses

from bearychat import Incoming, InvalidPayloadError


@pytest.fixture
def hook():
    return 'http://example.org'


@pytest.fixture
def incoming(hook):
    return Incoming(hook)


def test_text(incoming):
    expected_text = 'foobar'
    message = incoming \
        .reset() \
        .with_text(expected_text) \
        .build_message()
    assert message['text'] == expected_text
    assert message['markdown'] is False

    message = incoming \
        .reset() \
        .with_text(expected_text, markdown=True) \
        .build_message()
    assert message['text'] == expected_text
    assert message['markdown'] is True

    message = incoming \
        .reset() \
        .with_markdown(expected_text) \
        .build_message()
    assert message['text'] == expected_text
    assert message['markdown'] is True

    with pytest.raises(InvalidPayloadError):
        incoming.reset().build_message()


def test_to(incoming):
    message = incoming \
        .reset() \
        .with_text('foobar') \
        .build_message()
    assert 'channel' not in message

    expected_room = 'foobar'
    message = incoming \
        .reset() \
        .to(expected_room) \
        .with_text('foobar') \
        .build_message()
    assert message['channel'] == expected_room


def test_attachment(incoming):
    message = incoming \
        .reset() \
        .with_text('foobar') \
        .with_attachment(title='foobar') \
        .build_message()
    assert isinstance(message['attachments'], list)

    with pytest.raises(InvalidPayloadError):
        incoming \
            .reset() \
            .with_text('foobar') \
            .with_attachment(something_else='foobar') \
            .build_message()


@responses.activate
def test_push(incoming, hook):
    responses.add(responses.POST, hook, status=200)

    incoming \
        .reset() \
        .with_text('foobar') \
        .push()
    content_type = responses.calls[0].request.headers['Content-Type']
    assert content_type == 'application/json'



# coding: utf-8

import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


install_requires = [
    'requests>=2.4.2',
]


class PyTest(TestCommand):

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


setup(
    name='bearychat-py',
    version='0.3.1',
    author='hbc',
    author_email='bcxxxxxx@gmail.com',
    url='https://github.com/bcho/bearychat-py',
    description='A simple package for interacting with bearychat.',
    long_description=open('README.md').read(),
    license='MIT',
    packages=find_packages('.', exclude=['tests*']),
    zip_safe=False,
    install_requires=install_requires,

    cmdclass={'test': PyTest},
    tests_require=['pytest', 'responses'],

    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ]
)
